@@ -8,6 +8,7 @@ pub mod details;
 pub mod file_name;
 pub mod grid_details;
 pub mod grid;
+pub mod icons;
 pub mod lines;
 pub mod render;
 pub mod table;
@@ -29,9 +30,10 @@ pub struct View {
 
 /// The **mode** is the “type” of output.
 #[derive(Debug)]
+#[allow(clippy::large_enum_variant)]
 pub enum Mode {
     Grid(grid::Options),
     Details(details::Options),
     GridDetails(grid_details::Options),
-    Lines,
+    Lines(lines::Options),
 }
