@@ -1,13 +1,18 @@
 use datetime::TimeZone;
 use ansi_term::Style;
 
-use fs::fields as f;
 use output::cell::TextCell;
 use output::time::TimeFormat;
 
 
-impl f::Time {
-    pub fn render(self, style: Style,
+pub trait Render {
+    fn render(self, style: Style,
+                        tz: &Option<TimeZone>,
+                        format: &TimeFormat) -> TextCell;
+}
+
+impl Render for std::time::Duration {
+    fn render(self, style: Style,
                         tz: &Option<TimeZone>,
                         format: &TimeFormat) -> TextCell {
 
